
/* Chrysalide - Outil d'analyse de fichiers binaires
 * module.c - intégration du répertoire analysis en tant que module
 *
 * Copyright (C) 2010-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "module.h"


#include <assert.h>


#include "binary.h"
#include "block.h"
#include "content.h"
#include "routine.h"
#include "blocks/module.h"
#include "contents/module.h"
#include "db/module.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Ajoute le module 'analysis' au module Python.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool add_analysis_module_to_python_module(PyObject *super)
{
    bool result;                            /* Bilan à retourner           */
    PyObject *module;                       /* Sous-module mis en place    */
    int ret;                                /* Bilan d'un appel            */

    static PyModuleDef py_chrysalide_analysis_module = {

        .m_base = PyModuleDef_HEAD_INIT,

        .m_name = "pychrysalide.analysis",
        .m_doc = "Python module for Chrysalide.analysis",

        .m_size = -1,

    };

    result = false;

    module = PyModule_Create(&py_chrysalide_analysis_module);
    if (module == NULL) return false;

    ret = PyState_AddModule(super, &py_chrysalide_analysis_module);
    if (ret != 0) goto loading_failed;

    ret = _PyImport_FixupBuiltin(module, "pychrysalide.analysis");
    if (ret != 0) goto loading_failed;

    Py_INCREF(module);
    ret = PyModule_AddObject(super, "analysis", module);
    if (ret != 0) goto loading_failed;

    result = true;

    result &= register_python_loaded_binary(module);
    result &= register_python_instr_block(module);
    //result &= register_python_binary_content(module);
    result &= register_python_binary_routine(module);

    result &= add_analysis_blocks_module_to_python_module(module);
    result &= add_analysis_contents_module_to_python_module(module);
    result &= add_analysis_db_module_to_python_module(module);

 loading_failed:

    assert(result);

    return result;

}
