
/* Chrysalide - Outil d'analyse de fichiers binaires
 * ##FILE## - traduction d'instructions ARMv7
 *
 * Copyright (C) 2014 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


@title B

@encoding(t1) {

    @half 1 1 0 1 cond(4) imm8(8)

    @syntax {c} <label>

    @conv {

        c = Condition(cond)
        label = SignExtend(imm8:'0', imm8 & 0x80, 32)

    }

    @hooks {

        fetch = help_fetching_with_instruction_b_from_thumb
        link = handle_arm_conditional_branch_as_link
        post = post_process_branch_instructions

    }

    @rules {

        //if cond == '1110' then UNDEFINED;
        //if cond == '1111' then SEE SVC;
        //chk_call SetInsFlag(AIF_RETURN_POINT)

    }

}

@encoding(t2) {

    @half 1 1 1 0 0 imm11(11)

    @syntax <label>

    @conv {

        label = SignExtend(imm11:'0', imm11 & 0x400, 32)

    }

    @hooks {

        fetch = help_fetching_with_instruction_b_from_thumb
        link = handle_arm_conditional_branch_as_link
        post = post_process_branch_instructions

    }

    @rules {

        //if InITBlock() && !LastInITBlock() then UNPREDICTABLE;
        //chk_call SetInsFlag(AIF_RETURN_POINT)

    }

}

@encoding(T3) {

    @word 1 1 1 1 0 S(1) cond(4) imm6(6) 1 0 J1(1) 0 J2(1) imm11(11)

    @syntax {c} ".W" <label>

    @conv {

        c = Condition(cond)
        label = SignExtend(S:J2:J1:imm6:imm11:'0', S, 32)

    }

    @hooks {

        fetch = help_fetching_with_instruction_b_from_thumb
        link = handle_arm_conditional_branch_as_link
        post = post_process_branch_instructions

    }

    @rules {

        //if cond<3:1> == '111' then SEE "Related encodings";
        //if InITBlock() then UNPREDICTABLE;
        //chk_call SetInsFlag(AIF_RETURN_POINT)

    }

}

@encoding(T4) {

    @word 1 1 1 1 0 S(1) imm10(10) 1 0 J1(1) 1 J2(1) imm11(11)

    @syntax "b.W" <label>

    @conv {

        I1 = NOT(J1 EOR S)
        I2 = NOT(J2 EOR S)
        label = SignExtend(S:I1:I2:imm10:imm11:'0', S, 32)

    }

    @hooks {

        fetch = help_fetching_with_instruction_b_from_thumb
        link = handle_arm_conditional_branch_as_link
        post = post_process_branch_instructions

    }

    @rules {

        //if InITBlock() && !LastInITBlock() then UNPREDICTABLE;
        //chk_call SetInsFlag(AIF_RETURN_POINT)

    }

}

@encoding(A1) {

    @word cond(4) 1 0 1 0 imm24(24)

    @syntax {c} <label>

    @conv {

        c = Condition(cond)
        label = SignExtend(imm24:'00', imm24 & 0x800000, 32)

    }

    @hooks {

        fetch = help_fetching_with_instruction_b_from_arm
        link = handle_arm_conditional_branch_as_link
        post = post_process_branch_instructions

    }

    @rules {

        //chk_call SetInsFlag(AIF_RETURN_POINT)

    }

}
