
/* Chrysalide - Outil d'analyse de fichiers binaires
 * ##FILE## - traduction d'instructions ARMv7
 *
 * Copyright (C) 2014 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


@title LDRB (register)

@encoding(t1) {

    @half 0 1 0 1 1 1 0 Rm(3) Rn(3) Rt(3)

    @syntax <Rgt> <access>

    @conv {

        Rgt = Register(Rt)
        Rgn = Register(Rn)
        Rgm = Register(Rm)
        access = _MakeMemoryAccess(Rgn, Rgm, 0)

    }

    @rules {

    }

}

@encoding(T2) {

    @word 1 1 1 1 1 0 0 0 0 0 0 1 Rn(4) Rt(4) 0 0 0 0 0 0 imm2(2) Rm(4)

    @syntax "ldrb.W" <Rgt>, <access>

    @conv {

        Rgt = Register(Rt)
        Rgn = Register(Rn)
        Rgm = Register(Rm)
        shift = DecodeImmShift(0, imm2)
        access = MakeShiftedMemoryAccess(Rgn, Rgm, shift, 0)

    }

    @rules {

        //if Rn == '1111' then UNDEFINED;
        //if t IN {13,15} || m IN {13,15} then UNPREDICTABLE;

        if (Rt == '1111'); chk_call SetInsFlag(AIF_RETURN_POINT)

    }

}

@encoding(A11) {

    @word cond(4) 0 1 1 1 U(1) 1 W(1) 1 Rn(4) Rt(4) imm5(5) type(2) 0 Rm(4)

    @syntax <Rgt> <access>

    @conv {

        Rgt = Register(Rt)
        Rgn = Register(Rn)
        Rgm = Register(Rm)
        shift = DecodeImmShift(type, imm5)
        access = MakeShiftedMemoryAccess(Rgn, Rgm, shift, W)

    }

    @rules {

        //if P == '0' && W == '1' then SEE LDRBT;
        //if t == 15 || m == 15 then UNPREDICTABLE;
        //if wback && (n == 15 || n == t) then UNPREDICTABLE;
        //if ArchVersion() < 6 && wback && m == n then UNPREDICTABLE;
        if (Rt == '1111'); chk_call SetInsFlag(AIF_RETURN_POINT)

    }

}

@encoding(A12) {

    @word cond(4) 0 1 1 0 U(1) 1 W(1) 1 Rn(4) Rt(4) imm5(5) type(2) 0 Rm(4)

    @syntax <Rgt> <base> <offset> <?shift>

    @conv {

        Rgt = Register(Rt)
        Rgn = Register(Rn)
        Rgm = Register(Rm)
        base = MakeMemoryNotIndexed(Rgn, 1)
        offset = MakeAccessOffset(U, Rgm)
        shift = DecodeImmShift(type, imm5)

    }

    @rules {

        //if P == '0' && W == '1' then SEE LDRBT;
        //if t == 15 || m == 15 then UNPREDICTABLE;
        //if wback && (n == 15 || n == t) then UNPREDICTABLE;
        //if ArchVersion() < 6 && wback && m == n then UNPREDICTABLE;
        if (Rt == '1111'); chk_call SetInsFlag(AIF_RETURN_POINT)

    }

}
