
/* Chrysalide - Outil d'analyse de fichiers binaires
 * op_push.c - décodage des empilements
 *
 * Copyright (C) 2008-2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "opcodes.h"


#include "../operand.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : data   = flux de données à analyser.                         *
*                pos    = position courante dans ce flux. [OUT]               *
*                len    = taille totale des données à analyser.               *
*                addr   = adresse virtuelle de l'instruction.                 *
*                prefix = éventuel(s) préfixe(s) remarqué(s).                 *
*                proc   = architecture ciblée par le désassemblage.           *
*                                                                             *
*  Description : Décode une instruction de type 'push' (16 ou 32 bits).       *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchInstruction *x86_read_instr_push_imm1632(const bin_t *data, off_t *pos, off_t len, vmpa_t addr, X86Prefix prefix, const GX86Processor *proc)
{
    GArchInstruction *result;               /* Instruction à retourner     */
    MemoryDataSize oprsize;                 /* Taille des opérandes        */

    result = g_x86_instruction_new(XOP_PUSH_IMM1632);

    oprsize = g_x86_processor_get_operand_size(proc, prefix);

    if (!x86_read_one_operand(result, data, pos, len, X86_OTP_IMM1632, oprsize))
    {
        /* TODO free(result);*/
        return NULL;
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : data   = flux de données à analyser.                         *
*                pos    = position courante dans ce flux. [OUT]               *
*                len    = taille totale des données à analyser.               *
*                addr   = adresse virtuelle de l'instruction.                 *
*                prefix = éventuel(s) préfixe(s) remarqué(s).                 *
*                proc   = architecture ciblée par le désassemblage.           *
*                                                                             *
*  Description : Décode une instruction de type 'push' (16 ou 32 bits).       *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchInstruction *x86_read_instr_push_r1632(const bin_t *data, off_t *pos, off_t len, vmpa_t addr, X86Prefix prefix, const GX86Processor *proc)
{
    GArchInstruction *result;               /* Instruction à retourner     */
    X86Opcodes opcode;                      /* Instruction effective       */
    MemoryDataSize oprsize;                 /* Taille des opérandes        */

    opcode = XOP_PUSH_E_AX + (data[*pos] - 0x50);

    result = g_x86_instruction_new(opcode);

    oprsize = g_x86_processor_get_operand_size(proc, prefix);

    if (!x86_read_one_operand(result, data, pos, len, X86_OTP_OP_R1632, oprsize, 0x50))
    {
        /* TODO free(result);*/
        return NULL;
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : data = flux de données à analyser.                           *
*                pos  = position courante dans ce flux. [OUT]                 *
*                len  = taille totale des données à analyser.                 *
*                addr = adresse virtuelle de l'instruction.                   *
*                proc = architecture ciblée par le désassemblage.             *
*                                                                             *
*  Description : Décode une instruction de type 'push' (16 ou 32 bits).       *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchInstruction *x86_read_instr_push_rm1632(const bin_t *data, off_t *pos, off_t len, vmpa_t addr, X86Prefix prefix, const GX86Processor *proc)
{
    GArchInstruction *result;               /* Instruction à retourner     */
    MemoryDataSize oprsize;                 /* Taille des opérandes        */

    result = g_x86_instruction_new(XOP_PUSH_RM1632);

    oprsize = g_x86_processor_get_operand_size(proc, prefix);

    if (!x86_read_one_operand(result, data, pos, len, X86_OTP_RM1632, oprsize))
    {
        /* TODO free(result);*/
        return NULL;
    }

    return result;

}
